package jeu;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/**
 * Un objet graphique qui affiche un {@code Plateau} et permet de recueillir
 * les clics utilisateurs. Ceci est une implantation de l'interface
 * {@code IOPlateau}.
 *
 * @author Pierre.Courtieu at univ-orleans.fr
 * @author Jeremy.Briffaut at ensi-bourges.fr
 * @version 1.0
 */
public class IOGraphiquePlateau implements IOPlateau {

    private JFrame jframe; // La fenêtre

    private MonComp comp; // Le composant graphique damier

    private Plateau plateau; // Les pièces du plateau, une piece vide pour les case vide.
    
    private Coordonnees lastPositionClicked = new Coordonnees(-1, -1);
    
    private boolean click; // un clique met ce booleen à true, voir getCoup()

    private int width=0;
    private int height = 0;

    /**
     * Construit un nouvel affichage pour le plateau {@code p}, toute
     * modification de {@code p} dans la suite sera affichée au prochain
     * appel à la méthode {@code getcoup()}.
     */
    public IOGraphiquePlateau(Plateau p) {
        this.plateau = p;
        jframe = new JFrame("Jeu De Plateau");
        comp = new MonComp();

        // Ceci est un morceau de code très laid, qui n'existe pas dans la
        // bonne implémentation. Pour des raisons pédagogiques je ne peux pas
        // faire autrement sans donner trop de clés sur la bonne façon de
        // concevoir tout le programme. Je laisse donc ceci ici.
        try {
            while(true) {
                Piece pce = p.getPiece(width,0);
                this.width += 1;
            }
        } catch (Exception e){}
        try {
            while(true) {
                Piece pce = p.getPiece(0,height);
                this.height += 1;
            }
        } catch (Exception e){}
        // Now width and height should be ok.

        jframe.setSize(32*width, 32*height);
        comp.addMouseListener(new MouseAdapter() {

            public void mouseClicked(MouseEvent e) {
                int x = e.getX() / 32;
                int y = e.getY() / 32;
                lastPositionClicked = new Coordonnees(x, y);
                click = true;
            }
        });
        jframe.addWindowListener(new Quitter());
        jframe.getContentPane().add(comp);
        jframe.setResizable(false);
        comp.setPreferredSize(new Dimension(32*width, 32*height));
        jframe.pack();
        jframe.setVisible(true);
    }

    

    // voir la documentation IOPlateau
    public Coordonnees getCoup(String s) {
        System.out.println(s);
        comp.repaint();
        click = false;
        while (!click) {
            try {
                Thread.sleep(100);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        return lastPositionClicked;
    }

    // Définition du composant graphique représentant le damier.
    private class MonComp extends JComponent {

        public void paint(Graphics g) {
            if (plateau != null) {
                int nbcols = width;
                int nblines = height;
                int k = 0;
                for (int j = 0; j < nblines; j = j + 1) {
                    k = j%2;
                    for (int i = 0; i < nbcols; i = i + 1) {
                        java.awt.Color bgcolor;
                        if (k == 0) {
                            bgcolor = java.awt.Color.lightGray;
                        } else {
                            bgcolor = new java.awt.Color(0, 120, 200);
                        }
                        Color[][] tmp = plateau.getPiece(i,j).getIcon(bgcolor);// tableau de pixels 32x32
                        int dim = tmp.length;
                        int facteurzoom = (int) 32 / dim;
                        int x = i * 32, y = j * 32;
                        for (int l = 0; l < dim; l = l + 1) {
                            for (int m = 0; m < dim; m = m + 1) {
                                g.setColor(tmp[l][m]);
                                int aux1 = x + facteurzoom * m;
                                int aux2 = y + facteurzoom * l;
                                g.fillRect(aux1, aux2, aux1 + (facteurzoom - 1),
                                        aux2 + (facteurzoom - 1));
                            }
                        }
                        k = (k + 1) % 2;
                    }
                    //k = (k + 1) % 2;
                }
            }
        }
    }

    // Gestion de l'événement "clique sur le bouton de fermeture de la fenêtre.
    private class Quitter extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        }
    }
}
/*
 *** Local Variables: ***
 *** tab-width: 4 ***
 *** compile-command: "cd ../.. ; javac -d classes -sourcepath src/ src/jeuplateau/IOGraphiquePlateau.java" ***
 *** End: ***
 */
